package org.apache.maven.continuum;

import org.apache.maven.continuum.model.project.ProjectDependency;
import org.apache.maven.continuum.project.DependencyGroup;
import org.codehaus.plexus.logging.AbstractLogEnabled;

import java.util.*;

/**
 * User: drolsham
 */
public class DefaultDependencyHelper extends AbstractLogEnabled implements DependencyHelper {
    /**
     * This method finds all possible ways of creating a valid set of dependencies.
     * I.e., all sets that have identical <code>artifactId</code> and <code>groupId</code>
     * as the dependencies from <code>originalDependencies</code>.
     *
     *
     * @param originalDependencies  the base set of dependencies
     * @param newDependencies   the set to choose replacements from
     * @return  a list of lists containing the valid combinations
     */
    public List<List<ProjectDependency>> getAllDependencyCombinations(
            final List<ProjectDependency> originalDependencies,
            List<ProjectDependency> newDependencies) {

        //getLogger().info("Starting getAllDependencyCombinations");
        List<ProjectDependency> cleanedDependencies =
                findRelevantDependencies(originalDependencies, newDependencies);

        List<ProjectDependency> unaffectedDependencies =
                        extractUnaffectedDependencies(originalDependencies, cleanedDependencies);

        //originalDependencies =  cleanedDependencies +  extractUnaffectedDependencies       

        List<DependencyGroup> groups = makeDependencyGroups(cleanedDependencies);

        List<List<ProjectDependency>> combinations = new ArrayList<List<ProjectDependency>>();
        Vector<ProjectDependency> selection = new Vector<ProjectDependency>(groups.size());
        selection.setSize(groups.size());  //Must ensure that totalt _size_ is equal or greater than number of buckets.
        dependencyCombinations(0, groups, selection, combinations);

        if (getLogger().isInfoEnabled()) {
            for (List<ProjectDependency> list : combinations) {
                if (list.isEmpty()) {
                    getLogger().info("The list was empty");
                } else {
                    getLogger().info("");
                    getLogger().info("A new combination");
                    for (ProjectDependency dependencyElement : list) {

                        getLogger().info(dependencyElement.getGroupId() + ":"
                                        + dependencyElement.getArtifactId() + ":"
                                        + dependencyElement.getVersion());
                    }
                }

            }
            getLogger().info(combinations.size() + " combination(s) was found.");
        }
        
        //Append the rest of the originalDependencies to the results.
        List<List<ProjectDependency>> result = new ArrayList<List<ProjectDependency>>();
        for (List<ProjectDependency> list : combinations) {            
            list.addAll(cloneDependencyList(unaffectedDependencies));                        
            result.add(list);
            
            if (getLogger().isDebugEnabled()) {
                getLogger().debug(list.size() + " dependencies was added to the result. "
                        + unaffectedDependencies.size() + " was not unaffected.");

            }
        }
        return result;
    }

    protected List<ProjectDependency> findRelevantDependencies(
               final List<ProjectDependency> originalDependencies,
               final List<ProjectDependency> newDependencies) {

         List<ProjectDependency> result = new ArrayList<ProjectDependency>();

        if (newDependencies == null) {
            return result;
        }
        getLogger().info("Size before cleaning: " + newDependencies.size());
        //cleaning newDependencies for non-relevant dependencies
        boolean relevant = false;
        for (ProjectDependency dependencyElement : newDependencies) {
            //dependencyElement.setDerived(true);

            getLogger().info("dependencyElement: "
                                 + dependencyElement.getGroupId() + ":"
                                 + dependencyElement.getArtifactId() + ":"
                                 + dependencyElement.getVersion()
                                 + " derived: " + dependencyElement.isDerived());

            List<ProjectDependency> existing = findSimilarDependenciesInList(cloneDependency(dependencyElement), originalDependencies);
            //getLogger().info("Number of similar dependencies " + existing.size());
            if (existing != null && !existing.isEmpty()) {
                for (ProjectDependency element : existing) {
                    if (!result.contains(element)) {
                        result.add(element);                       
                    }
                }
                /*else {
                    //TODO throw programmingerrorexception
                    getLogger().error("ProgrammingError: " +
                            "This code assumes that the default set only have ONE dependency of each type.");
                }*/

            }
        }
        getLogger().info("Number of relevant dependencies: " + result.size());
        return result;
    }


    public List<ProjectDependency> findSimilarDependenciesInList(
            final ProjectDependency dependency, final List<ProjectDependency> list) {

        List<ProjectDependency> result = new ArrayList<ProjectDependency>();
        for  (ProjectDependency element : list) {            
            if (element.getArtifactId().equals(dependency.getArtifactId()) &&
                    element.getGroupId().equals(dependency.getGroupId())) {

                if (!result.contains(dependency)) {
                    result.add(dependency);
                }                
                result.add(element);
            }
        }        
        return result;
    }

      public List<ProjectDependency> findEqualDependenciesInList(final ProjectDependency dependency,
                                final List<ProjectDependency> list) {

        List<ProjectDependency> result = new ArrayList<ProjectDependency>();
        for  (ProjectDependency element : list) {
            if (element.getArtifactId().equals(dependency.getArtifactId()) &&
                    element.getGroupId().equals(dependency.getGroupId())) {
               result.add(element);
               result.add(dependency);
            }
        }
        return result;
    }

    protected List<DependencyGroup> makeDependencyGroups(List<ProjectDependency> newDependencies) {
        //getLogger().info("Starting makeDependencyGroups");
        getLogger().info(newDependencies.size() + " dependencies to put into groups.");
        List<DependencyGroup> groups = new ArrayList<DependencyGroup>();
        for (ProjectDependency dependencyElement : newDependencies) {
            if (dependencyElement != null) {
                //dependencyElement.setDerived(true);
            } else {
               getLogger().info("dependencyElement is NULL ");
            }
            ListIterator<DependencyGroup> groupsIterator = groups.listIterator();
            if (!groupsIterator.hasNext()) {
                //getLogger().info("No groups exist.");
                addToGroupList(dependencyElement, groupsIterator);                
            } else {
                boolean exist = false;
                DependencyGroup groupElement;
                while (groupsIterator.hasNext()) {
                    groupElement = groupsIterator.next();
                    if (dependencyElement.getArtifactId().equals(groupElement.getArtifactId()) &&
                            dependencyElement.getGroupId().equals(groupElement.getGroupId())) {

                        groupElement.add(dependencyElement);
                        groupsIterator.set(groupElement);                        
                        exist = true;
                    } 
                }
                if (!exist) {;
                    addToGroupList(dependencyElement, groupsIterator);    
                }
            }
        }
        getLogger().info("groups.size(): " + groups.size());
        return groups;
    }


     /**
     * This method combines one element from each <code>group</code>
     * with all combinations of elements from the other groups.
     *
     * <code>dependencyCombinations</code> is a recursive algorithm which will
     * generate all possible ways of completing the <code>selection</code>, given
     * <code>currentGroup</code> and a <code>selection</code>
     * populated up to index < <code>currentGroup</code> -1.
     *
     *
     * @param currentGroup the index of the bucket we are currently operating on
     * @param groups   the <code>DependencyGroups</code> to choose dependencies from
     * @param selection list of elements chosen so far. Note! The size must be equal to the size of groups.
     * @param uniqueCombinations    the list of lists containing valid combinations
     */
    private void dependencyCombinations(int currentGroup,
                                    final List<DependencyGroup> groups,
                                    List<ProjectDependency> selection,
                                    List<List<ProjectDependency>> uniqueCombinations) {

        if (currentGroup == groups.size()) {
            uniqueCombinations.add(cloneDependencyList(selection));
            //We've been through all groups and have a new uniqueCombination            
        } else {
            ProjectDependency projectDependency;
            DependencyGroup group = groups.get(currentGroup);
            for (int i = 0; i < group.size(); i++) {
                projectDependency = group.get(i);
                selection.set(currentGroup, projectDependency);                 
                dependencyCombinations(currentGroup + 1, groups, selection, uniqueCombinations);  
            }
        }
    }

    protected List<ProjectDependency> extractUnaffectedDependencies(
              final List<ProjectDependency> originalDependencies,
              final List<ProjectDependency> cleanedDependencies) {

        getLogger().info("Extracting unaffectedDependencies - "
                + "originalDependencies.size: " + originalDependencies.size()
                + " cleanedDependencies.size: " + cleanedDependencies.size());

        List<ProjectDependency> result = new ArrayList<ProjectDependency>();
        for (ProjectDependency element : originalDependencies) {


            List<ProjectDependency> similarDependencies =
                    findSimilarDependenciesInList(element, cleanedDependencies);
            
            if (similarDependencies == null || similarDependencies.isEmpty())  {
                result.add(element);
                //getLogger().info("added");
            } else {
                getLogger().info(element.getGroupId() + ":"
                        + element.getArtifactId() + ":"
                        + element.getVersion()
                        + " was affected (i.e. not added to the not-affected-list)");
            }

        }
        getLogger().info(result.size() + " unaffectedDependencies");
        return result;
    }


    private void addToGroupList(ProjectDependency newDependency, ListIterator<DependencyGroup> listIterator) {
        DependencyGroup group = new DependencyGroup();
        group.add(newDependency);
        listIterator.add(group);
        /*getLogger().info("Creating new dependencyGroup and adding dependency : "
                + newDependency.getGroupId() + ":"
                + newDependency.getArtifactId() + ":"
                + newDependency.getVersion());*/
    }    
    

    /**
    * Convenience method when only one dependency is changed
    * @param originalDependencies
    * @param newDependency
    * @return
    * @deprecated
    */
   public List<ProjectDependency> createNewDependencyList(
           final List<ProjectDependency> originalDependencies, ProjectDependency newDependency) {

       List<ProjectDependency> updatedDependencies = new ArrayList<ProjectDependency>();
       String artifactId = newDependency.getArtifactId();
       String groupId = newDependency.getGroupId();
       boolean modified = false;

       for(ProjectDependency oldDependency : originalDependencies) {
           if( (oldDependency.getArtifactId().equals(artifactId)) &&
                   (oldDependency.getGroupId().equals(groupId)) ) {
               //System.out.println("++++ Modifying a dependency ++++");
               newDependency.setDerived(true);
               updatedDependencies.add(cloneDependency(newDependency));
               modified = true;
//                System.out.println("A dependency was modified  and its " +
//                        "artifactId is " + newDependency.getArtifactId());
               if (newDependency.isDerived()) {
                   System.out.println("derived: " + newDependency.isDerived());
               }
           } else {
               updatedDependencies.add(cloneDependency(oldDependency));
           }
       }
       if (!modified) {
           updatedDependencies = null;
       }
       return updatedDependencies;
   }
     
     /**
     * Creates an updated dependencylist with the new versions.
     * Operates on Continuum's <code>ProjectDependency</code>
     * @param originalDependencies
     * @param newDependencies
     * @return
     * @deprecated
     */
    public List<ProjectDependency> createNewDependencyList(
           List<ProjectDependency> originalDependencies,
           final List<ProjectDependency> newDependencies) {

        List<ProjectDependency> result = new ArrayList<ProjectDependency>();
        boolean modified = false;
        for(ProjectDependency oldDependency : originalDependencies) {
            for(ProjectDependency newDependency: newDependencies) {
                if( (oldDependency.getArtifactId().equals(newDependency.getArtifactId())) &&
                    (oldDependency.getGroupId().equals(newDependency.getGroupId())) ) {

                    newDependency.setDerived(true);
                    result.add(newDependency);
                    modified = true;
                } else {
                    result.add(oldDependency);
                }
            }
        }
        if (!modified) {
            result = null;
        }
        return result;
    }

    public boolean dependenciesEqual(ProjectDependency dependency1, ProjectDependency dependency2) {
        if (dependency1.getArtifactId().equals(dependency2.getArtifactId()) &&
                dependency1.getGroupId().equals(dependency2.getGroupId()) &&
                dependency1.getVersion().equals(dependency2.getVersion())) {
            return true;
        } else {
            return false;
        }
    }

    public List<ProjectDependency> cloneDependencyList(List<ProjectDependency> dependencies) {
        List<ProjectDependency> result = new ArrayList<ProjectDependency>();
        for (ProjectDependency element : dependencies) {
            result.add(cloneDependency(element));
        }
        return result;
    }
    
    public ProjectDependency cloneDependency (ProjectDependency dependency) {
        ProjectDependency result = new ProjectDependency();
        result.setArtifactId(dependency.getArtifactId());
        result.setGroupId(dependency.getGroupId());
        result.setVersion(dependency.getVersion());
        result.setModelEncoding(dependency.getModelEncoding());
        result.setDerived(dependency.isDerived());
        return result;
    }
}
