package org.apache.maven.continuum;

import org.apache.maven.continuum.model.project.*;
import org.apache.maven.continuum.store.ContinuumStore;

import org.codehaus.plexus.logging.AbstractLogEnabled;
import org.codehaus.plexus.logging.Logger;

import java.util.*;


/**
 * @author Erik Drolshammer 
 */
public class DefaultDerivedProjectManager extends AbstractLogEnabled implements DerivedProjectManager {
    /**
     * @plexus.requirement
     */
    private DependencyHelper dependencyHelper;

    /*
     * @plexus.requirement
     */
    //private ContinuumStore store;

        /*
    public DefaultDerivedProjectManager(ContinuumStore store) {
        this.store = store;
        //this.dependencyHelper = new DefaultDependencyHelper();
    }
    */

    public boolean projectExist(Project project, ContinuumStore store) {
        List<Project> projects = store.getAllProjectsWithAllDetails();
        for (Project element : projects) {
            if (projectSame(project, element)) {
                return true;
            }
        }
        return false;
    }

    //http://svn.apache.org/repos/asf/maven/components/trunk/maven-model/src/main/mdo/maven.mdo
    public List<Project> createDerivedProjects(final Project originalProject,
                                               List<ProjectDependency> newDependencies, ContinuumStore store) {

        return updateDerivedProjects(originalProject, newDependencies, null, store);
    }

    public List<Project> updateDerivedProjects(final Project originalProject,
                                               List<ProjectDependency> newDependencies,
                                               List<Project> derivedProjects, ContinuumStore store) {

        List<Project> result = new ArrayList<Project>();

        List<List<ProjectDependency>> allCombinations = dependencyHelper.
                getAllDependencyCombinations(originalProject.getDependencies(), newDependencies);
        

        Project derivedProject;
        for (int i = 0; i<allCombinations.size(); i++) {
            List<ProjectDependency> aCombination = allCombinations.get(i);
           
            derivedProject = findProjectWithSameDependencies(derivedProjects, aCombination);
            if (derivedProject == null) {
                derivedProject = new Project();
                derivedProject.setDerived(true);
                //TODO need the reference to the originalProject
                //derivedProject.setOriginalProject(originalProject);
                //getLogger().info("a new derivedProject was created");
                copyAttributesFromOriginalProject(originalProject, derivedProject);
                derivedProject.setDependencies(aCombination);
                
                if (!projectSame(originalProject, derivedProject)) {
                    getLogger().debug("version before: " + derivedProject.getVersion());
                    derivedProject.setVersion(derivedProject.getVersion() + "-" + i);
                    getLogger().debug("version after: " + derivedProject.getVersion());
                    result.add(derivedProject);
                } else {
                    getLogger().info("The original project has this combination of dependencies. No new project added.");
                    derivedProject = null;
                    //derivedProject.setDerived(false); //a project with the same dependencies already exists. 
                }
            } else {
                getLogger().info("Derived project with same dependencies already exist");
            }
        }        
        return result;        
    }


    protected Project findProjectWithSameDependencies(List<Project> projects,
                                                      List<ProjectDependency> dependencies) {

        if (projects != null) {
             for (Project project : projects) {
                if (project.getDependencies().contains(dependencies) &&
                        dependencies.contains(project.getDependencies())) {

                    return project;
                }
            }   
        }
        return null;
    }

    public void copyAttributesFromOriginalProject( Project originalProject, Project project ) {
        project.setGroupId( originalProject.getGroupId() );
        project.setArtifactId( originalProject.getArtifactId() );
        project.setExecutorId( originalProject.getExecutorId() );
        project.setName( originalProject.getName());
        project.setDescription( originalProject.getDescription() );
        project.setVersion( originalProject.getVersion() );

        project.setWorkingDirectory( originalProject.getWorkingDirectory() );
        project.setUrl(originalProject.getUrl());
        project.setScmUrl(originalProject.getScmUrl());
        project.setScmTag(originalProject.getScmTag());
        project.setScmUsername(originalProject.getScmUsername());
        project.setScmPassword(originalProject.getScmPassword());

        List<ProjectDeveloper> developers = new ArrayList<ProjectDeveloper>();
        List originalDevelopers = originalProject.getDevelopers();         
        ProjectDeveloper developer;
        for (int i = 0; i<originalDevelopers.size(); i++) {
            developer = (ProjectDeveloper) originalDevelopers.get(i);
            developers.add(cloneDeveloper(developer));
        }
        project.setDevelopers(developers);


        List<ProjectNotifier> notifiers = new ArrayList<ProjectNotifier>();
        List<ProjectNotifier> originalNotifiers = originalProject.getNotifiers();
        for (ProjectNotifier element : originalNotifiers) {
             notifiers.add(cloneNotifier(element));
        }
        project.setNotifiers(notifiers);

        List<BuildDefinition> buildDefinitions = new ArrayList<BuildDefinition>();
        List<BuildDefinition> originalBuildDefinitions = originalProject.getBuildDefinitions();
        for (BuildDefinition element : originalBuildDefinitions) {
             buildDefinitions.add(cloneBuildDefinition(element));
        }
        project.setBuildDefinitions(buildDefinitions);


        ProjectDependency parent = originalProject.getParent();
        if (parent != null) {
            project.setParent(dependencyHelper.cloneDependency(parent));    
        }

        // TODO are these relevant to copy? 
//        project.setCheckoutResult(originalProject.getCheckoutResult());
//        project.setProjectGroup(originalProject.getProjectGroup());

        //Not applicable
        //project.setBuildResults(originalProject.getBuildResults());
        //project.setState(originalProject.getState());
        //project.setOldState(originalProject.getOldState());
        //project.setLatestBuildId(originalProject.getLatestBuildId());
        //project.setBuildNumber(originalProject.getBuildNumber());
        //project.setId(id);
        //project.setDependencies(originalProject.getDependencies());
    }

     
    public boolean projectSame(Project project1, Project project2) {
        boolean result = true;

        if (!(project1.getGroupId().equals(project2.getGroupId()))) {
            return false;
        }
        if (!(project1.getArtifactId().equals(project2.getArtifactId()))) {
            return false;
        }
        if (!(project1.getVersion().equals(project2.getVersion()))) {
            return false;
        }        
        if (!(project1.getScmUrl().equals(project2.getScmUrl()))) {
            return false;
        }
        //getLogger().info("So far the two projects seem the same.");

        List<ProjectDependency> dependencies1 = project1.getDependencies();
        List<ProjectDependency> dependencies2 = project2.getDependencies();
        if (dependencies1.size() != dependencies2.size()) {
            getLogger().info("The dependencylists are not equally long.");
            return false;
        }

        ProjectDependency projectDependency1;
        ProjectDependency projectDependency2;
        boolean equal = false;
        for (Object element1 : dependencies1) {
            projectDependency1 = (ProjectDependency) element1;

            for (Object element2 : dependencies2) {
                projectDependency2 = (ProjectDependency) element2;

                if (projectDependency1.getGroupId().equals(projectDependency2.getGroupId()) &&
                        projectDependency1.getArtifactId().equals(projectDependency2.getArtifactId()) &&
                        projectDependency1.getVersion().equals(projectDependency2.getVersion())) {

                    equal = true;
                }
            }
            if (!equal) {
                return false;
            }
            equal = false;
        }
        return result;
    }

        

    public ProjectDeveloper cloneDeveloper (ProjectDeveloper developer) {
        ProjectDeveloper result = new ProjectDeveloper();
        result.setContinuumId(developer.getContinuumId());
        result.setEmail(developer.getEmail());
        result.setModelEncoding(developer.getModelEncoding());        
        result.setName(developer.getName());
        result.setScmId(developer.getScmId());              
        return result;
    }

    public ProjectNotifier cloneNotifier (ProjectNotifier notifier) {
        ProjectNotifier result = new ProjectNotifier();
        result.setConfiguration(notifier.getConfiguration());
        result.setFrom(notifier.getFrom());
        result.setModelEncoding(notifier.getModelEncoding());
        result.setRecipientType(notifier.getRecipientType());        
        result.setType(notifier.getType());
        result.setSendOnSuccess(notifier.isSendOnSuccess());
        result.setSendOnFailure(notifier.isSendOnFailure());
        result.setSendOnWarning(notifier.isSendOnWarning());
        result.setSendOnError(notifier.isSendOnError());
        return result;
    }

    public BuildDefinition cloneBuildDefinition (BuildDefinition buildDefinition) {
        BuildDefinition result = new BuildDefinition();
        result.setArguments(buildDefinition.getArguments());
        result.setBuildFile(buildDefinition.getBuildFile());
        result.setGoals(buildDefinition.getGoals());
        result.setLatestBuildId(buildDefinition.getLatestBuildId());        
        result.setModelEncoding(buildDefinition.getModelEncoding());
        result.setProfile(buildDefinition.getProfile());
        result.setSchedule(buildDefinition.getSchedule());
        result.setDefaultForProject(buildDefinition.isDefaultForProject());
        return result;
    }

    public void printProjectInfo(Logger logger, final Project project) {
        if (project == null) {
            System.out.println("project was null...");
            return;
        }
        if (logger == null) {
            logger = getLogger();
        }

        logger.debug("projectId: " + project.getId());
        logger.debug("projectName: " + project.getName());
        logger.debug("groupId: " + project.getGroupId());
        logger.debug("ArtifactId: " + project.getArtifactId());
        logger.debug("Version: " + project.getVersion());
        logger.debug("Derived: " + project.isDerived());
        //logger.debug("Description: " + project.getDescription());

        List<ProjectDependency> dependencies = project.getDependencies();
        if (dependencies.size() > 0) {
            logger.debug("=== #" + dependencies.size() + " dependencies ===");
            //logger.info("");
            /*
            for (ProjectDependency dependency : dependencies) {
                logger.debug(dependency.getGroupId() + ":"
                        + dependency.getArtifactId()
                        + ":" + dependency.getVersion()
                        + " - derived=" + dependency.isDerived());
                                
            }
            */
        } else {
            logger.debug("No dependencies was set");
        }
        logger.debug("");

        List<BuildDefinition> buildDefinitions = project.getBuildDefinitions();
        logger.debug("# of buildDefinitions: " + buildDefinitions.size());
        //logger.debug("===BuildDefinitions=== ");
        for (BuildDefinition element : buildDefinitions) {
             logger.debug("ID: " + element.getId()
                     + " isDefault: " + element.isDefaultForProject() 
                     + " goals: " + element.getGoals()
                     + " buildFile: " + element.getBuildFile());            
        }
    }

    public void printInfoForAllProjects(Logger logger, ContinuumStore store) {
        if (logger == null) {
            logger = getLogger();
        }
        List<Project> projects = store.getAllProjectsWithAllDetails();
        Project element;
        logger.debug("");
        logger.debug("Looping through all projects in store. Size=" + projects.size());
        logger.debug("");
        for (int i=0; i<projects.size(); i++) {
            //getLogger().info("Project nr " + i);
            element = (Project) projects.get(i);
            printProjectInfo(logger, element);
        }
    }

    public DependencyHelper getDependencyHelper() {
        return dependencyHelper;
    }
}
